//
// Run this program with
//
//    gunzip -c pythia8-10000dijets-ptmin500.UW.gz        | ./boosted-HZ dijets.hist
//    gunzip -c pythia8-200HZ-10000dijets-ptmin500.UW.gz  | ./boosted-HZ HZ+dijets.hist 
//
// to analyse a pure dijet sample and a sample with 2% of HZ events mixed in
//
// Then plot the results by opening gnuplot and typing
//
//    load 'plot.gp'
//
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include <cstdlib>
#include "fastjet/ClusterSequence.hh"
#include "EventSource.hh"
#include "print_jets.hh"
#include "SimpleHist.hh"
#include "AverageAndError.hh"
#include "fastjet/Selector.hh"
#include "fastjet/tools/MassDropTagger.hh"
#include "fastjet/tools/Filter.hh" 

using namespace std;
using namespace fastjet;


int main(int argc, char ** argv) {

  if (argc < 3) {
    cerr << "Usage: " << argv[0] << " event_file.PU14.gz outfile.hist [nev]" << endl;
    exit(-1);
  }
  string eventfile = argv[1];
  string outfile = argv[2];
  int maxiev = 10000;
  if (argc >= 4) maxiev = strtol(argv[3],0,0);

  EventSource event_source(eventfile);
  // max number of events read in (even if input file contains more)
  // If there are fewer events, it exits gracefully
  
  // CA, R=1.2
  JetDefinition jet_def(cambridge_algorithm,1.2);

  // initialize histograms (min, max, spacing)
  SimpleHist dijet_mass_hist(0.0, 4000.0, 100.0);  
  SimpleHist dijet_mass_hist_allcuts(0.0, 4000.0, 100.0);  
  int nev_pass_basic_cuts = 0;
  int nev_pass_all_cuts = 0;
  
  int iev=0;
  vector<PseudoJet> input_particles;
  
  while (++iev <= maxiev && event_source.next_event(input_particles)) {
    if (iev ==1 || iev%100 == 0) {
      cerr << "Event " << iev << ", size = " << input_particles.size() << endl; 
    }

    // run the jet clustering
    vector<PseudoJet> jets = jet_def(input_particles);

    // set up our basic cuts
    bool pass = true;
    double rapmax = 2.0;
    double ptmin = 400.0;
    
    // now check if the two hardest jets pass some hardness and rapidity cuts
    for (int i = 0; i < 2; i++) {
      if (jets[i].pt()       < ptmin ) pass = false;
      if (abs(jets[i].rap()) > rapmax) pass = false;
    }

    // don't bin anything if we don't pass the cuts
    if (!pass) continue;
    nev_pass_basic_cuts++;
    dijet_mass_hist.add_entry((jets[0]+jets[1]).m());

    // HERE YOU CAN PLACE AN EXTRA SET OF CUTS
    // (currently none)
    
    // then bin the results after the more advanced set of cuts
    if (!pass) continue;
    nev_pass_all_cuts++;
    dijet_mass_hist_allcuts.add_entry((jets[0]+jets[1]).m());
    
  }
    
  // open output file for histograms
  ofstream output_file;
  output_file.open(outfile.c_str());
      
  // output histograms
  output_file << "# jet_def: " << jet_def.description() << endl;
  output_file << "#" << endl;
  output_file << "# nev total = " << iev-1 << endl;  

  output_file << "# nev passing basic jet cuts = " << nev_pass_basic_cuts << endl;  
  output_file << "# dijet mass histogram (after basic jet cuts) " << endl;
  output(dijet_mass_hist,&output_file); // in SimpleHist.hh
  output_file << endl << endl; // double newline helps gnuplot separate histograms
  
  output_file << "# nev passing all jet cuts = " << nev_pass_all_cuts << endl;  
  output_file << "# dijet mass histogram (after all jet cuts) " << endl;
  output(dijet_mass_hist_allcuts,&output_file); // in SimpleHist.hh
  output_file << endl << endl;
  
  output_file.close();
  
}
